@@ -11,8 +11,7 @@ package Kernel::Output::HTML::DashboardCustomerCompanyInformation;
 use strict;
 use warnings;
 
-use Kernel::System::CustomerCompany;
-use Kernel::System::Valid;
+our $ObjectManagerDisabled = 1;
 
 sub new {
     my ( $Type, %Param ) = @_;
@@ -21,17 +20,11 @@ sub new {
     my $Self = {%Param};
     bless( $Self, $Type );
 
-    # get needed objects
-    for (
-        qw(Config Name ConfigObject LogObject DBObject LayoutObject ParamObject TicketObject UserID)
-        )
-    {
-        die "Got no $_!" if ( !$Self->{$_} );
+    # get needed parameters
+    for my $Needed (qw(Config Name UserID)) {
+        die "Got no $Needed!" if ( !$Self->{$Needed} );
     }
 
-    $Self->{CustomerCompanyObject} = Kernel::System::CustomerCompany->new( %{$Self} );
-    $Self->{ValidObject}           = Kernel::System::Valid->new( %{$Self} );
-
     $Self->{PrefKey} = 'UserDashboardPref' . $Self->{Name} . '-Shown';
 
     $Self->{CacheKey} = $Self->{Name};
@@ -62,23 +55,31 @@ sub Run {
 
     return if !$Param{CustomerID};
 
-    my %CustomerCompany = $Self->{CustomerCompanyObject}->CustomerCompanyGet(
+    my %CustomerCompany = $Kernel::OM->Get('Kernel::System::CustomerCompany')->CustomerCompanyGet(
         CustomerID => $Param{CustomerID},
     );
 
-    my $CustomerCompanyConfig = $Self->{ConfigObject}->Get( $CustomerCompany{Source} );
+    my $CustomerCompanyConfig = $Kernel::OM->Get('Kernel::Config')->Get( $CustomerCompany{Source} || '' );
     return if ref $CustomerCompanyConfig ne 'HASH';
     return if ref $CustomerCompanyConfig->{Map} ne 'ARRAY';
 
     return if !%CustomerCompany;
 
+    # get layout object
+    my $LayoutObject = $Kernel::OM->Get('Kernel::Output::HTML::Layout');
+    my $ValidObject  = $Kernel::OM->Get('Kernel::System::Valid');
+    my $CompanyIsValid;
+
     # make ValidID readable
     if ( $CustomerCompany{ValidID} ) {
-        $CustomerCompany{ValidID} = $Self->{ValidObject}->ValidLookup(
+        my @ValidIDs = $ValidObject->ValidIDsGet();
+        $CompanyIsValid = grep { $CustomerCompany{ValidID} == $_ } @ValidIDs;
+
+        $CustomerCompany{ValidID} = $ValidObject->ValidLookup(
             ValidID => $CustomerCompany{ValidID},
         );
 
-        $CustomerCompany{ValidID} = $Self->{LayoutObject}->{LanguageObject}->Get( $CustomerCompany{ValidID} );
+        $CustomerCompany{ValidID} = $LayoutObject->{LanguageObject}->Translate( $CustomerCompany{ValidID} );
     }
 
     ENTRY:
@@ -91,17 +92,17 @@ sub Run {
         # do not show empty entries
         next ENTRY if !length( $CustomerCompany{$Key} );
 
-        $Self->{LayoutObject}->Block( Name => "ContentSmallCustomerCompanyInformationRow" );
+        $LayoutObject->Block( Name => "ContentSmallCustomerCompanyInformationRow" );
 
         if ( $Key eq 'CustomerID' ) {
-            $Self->{LayoutObject}->Block(
+            $LayoutObject->Block(
                 Name => "ContentSmallCustomerCompanyInformationRowLink",
                 Data => {
                     %CustomerCompany,
                     Label => $Entry->[1],
                     Value => $CustomerCompany{$Key},
                     URL =>
-                        '$Env{"Baselink"}Action=AdminCustomerCompany;Subaction=Change;CustomerID=$QData{"CustomerID"};Nav=Agent',
+                        '[% Env("Baselink") %]Action=AdminCustomerCompany;Subaction=Change;CustomerID=[% Data.CustomerID | uri %];Nav=Agent',
                     Target => '',
                 },
             );
@@ -111,7 +112,7 @@ sub Run {
 
         # check if a link must be placed
         if ( $Entry->[6] ) {
-            $Self->{LayoutObject}->Block(
+            $LayoutObject->Block(
                 Name => "ContentSmallCustomerCompanyInformationRowLink",
                 Data => {
                     %CustomerCompany,
@@ -126,7 +127,7 @@ sub Run {
 
         }
 
-        $Self->{LayoutObject}->Block(
+        $LayoutObject->Block(
             Name => "ContentSmallCustomerCompanyInformationRowText",
             Data => {
                 %CustomerCompany,
@@ -134,9 +135,15 @@ sub Run {
                 Value => $CustomerCompany{$Key},
             },
         );
+
+        if ( $Key eq 'CustomerCompanyName' && defined $CompanyIsValid && !$CompanyIsValid ) {
+            $LayoutObject->Block(
+                Name => 'ContentSmallCustomerCompanyInvalid',
+            );
+        }
     }
 
-    my $Content = $Self->{LayoutObject}->Output(
+    my $Content = $LayoutObject->Output(
         TemplateFile => 'AgentDashboardCustomerCompanyInformation',
         Data         => {
             %{ $Self->{Config} },
