@@ -1,6 +1,5 @@
 # --
-# Kernel/Output/HTML/DashboardRSS.pm
-# Copyright (C) 2001-2013 OTRS AG, http://otrs.com/
+# Copyright (C) 2001-2017 OTRS AG, http://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
 # the enclosed file COPYING for license information (AGPL). If you
@@ -14,8 +13,7 @@ use warnings;
 
 use XML::FeedPP;
 
-use vars qw($VERSION);
-$VERSION = qw($Revision: 1.17 $) [1];
+our $ObjectManagerDisabled = 1;
 
 sub new {
     my ( $Type, %Param ) = @_;
@@ -24,12 +22,9 @@ sub new {
     my $Self = {%Param};
     bless( $Self, $Type );
 
-    # get needed objects
-    for (
-        qw(Config Name ConfigObject LogObject DBObject LayoutObject ParamObject TicketObject UserID)
-        )
-    {
-        die "Got no $_!" if ( !$Self->{$_} );
+    # get needed parameters
+    for my $Needed (qw(Config Name UserID)) {
+        die "Got no $Needed!" if ( !$Self->{$Needed} );
     }
 
     return $Self;
@@ -46,7 +41,9 @@ sub Config {
 
     return (
         %{ $Self->{Config} },
-        CacheKey => 'RSS' . $Self->{Config}->{URL} . '-' . $Self->{LayoutObject}->{UserLanguage},
+        CacheKey => 'RSS'
+            . $Self->{Config}->{URL} . '-'
+            . $Kernel::OM->Get('Kernel::Output::HTML::Layout')->{UserLanguage},
     );
 }
 
@@ -56,7 +53,10 @@ sub Run {
     # Default URL
     my $FeedURL = $Self->{Config}->{URL};
 
-    my $Language = $Self->{LayoutObject}->{UserLanguage};
+    # get layout object
+    my $LayoutObject = $Kernel::OM->Get('Kernel::Output::HTML::Layout');
+
+    my $Language = $LayoutObject->{UserLanguage};
 
     # Check if URL for UserLanguage is available
     if ( $Self->{Config}->{"URL_$Language"} ) {
@@ -71,45 +71,69 @@ sub Run {
         }
     }
 
-    # set proxy settings can't use Kernel::System::WebAgent because of used
-    # XML::FeedPP to get RSS files
-    my $Proxy = $Self->{ConfigObject}->Get('WebUserAgent::Proxy');
+    # Configure a local instance of LWP::UserAgent for passing to XML::FeedPP.
+    my $UserAgent = LWP::UserAgent->new();
+
+    my $ConfigObject = $Kernel::OM->Get('Kernel::Config');
+
+    # In some scenarios like transparent HTTPS proxies, it can be necessary to turn off
+    #   SSL certificate validation.
+    if ( $ConfigObject->Get('WebUserAgent::DisableSSLVerification') ) {
+        $UserAgent->ssl_opts(
+            verify_hostname => 0,
+        );
+    }
+
+    # Set proxy settings if configured, and make sure to allow all supported protocols
+    #   (please see bug#12512 for more information).
+    my $Proxy = $ConfigObject->Get('WebUserAgent::Proxy');
     if ($Proxy) {
-        $ENV{CGI_HTTP_PROXY} = $Proxy;
+        $UserAgent->proxy( [ 'http', 'https', 'ftp' ], $Proxy );
     }
 
     # get content
-    my $Feed = eval {
-        XML::FeedPP->new(
-            $FeedURL,
-            'xml_deref' => 1,
-            'utf8_flag' => 1,
-        );
-    };
+    my $Feed;
+
+    TRY:
+    for ( 1 .. 3 ) {
+        $Feed = eval {
+            XML::FeedPP->new(
+                $FeedURL,
+                'xml_deref'     => 1,
+                'utf8_flag'     => 1,
+                'lwp_useragent' => $UserAgent,
+            );
+        };
+        last TRY if $Feed;
+    }
 
     if ( !$Feed ) {
         my $Content = "Can't connect to $FeedURL";
         return $Content;
     }
 
+    # get time object
+    my $TimeObject = $Kernel::OM->Get('Kernel::System::Time');
+
     my $Count = 0;
+    ITEM:
     for my $Item ( $Feed->get_item() ) {
         $Count++;
-        last if $Count > $Self->{Config}->{Limit};
+        last ITEM if $Count > $Self->{Config}->{Limit};
         my $Time = $Item->pubDate();
         my $Ago;
         if ($Time) {
-            my $SystemTime = $Self->{TimeObject}->TimeStamp2SystemTime(
+            my $SystemTime = $TimeObject->TimeStamp2SystemTime(
                 String => $Time,
             );
-            $Ago = $Self->{TimeObject}->SystemTime() - $SystemTime;
-            $Ago = $Self->{LayoutObject}->CustomerAge(
+            $Ago = $TimeObject->SystemTime() - $SystemTime;
+            $Ago = $LayoutObject->CustomerAge(
                 Age   => $Ago,
                 Space => ' ',
             );
         }
 
-        $Self->{LayoutObject}->Block(
+        $LayoutObject->Block(
             Name => 'ContentSmallRSSOverviewRow',
             Data => {
                 Title => $Item->title(),
@@ -117,7 +141,7 @@ sub Run {
             },
         );
         if ($Ago) {
-            $Self->{LayoutObject}->Block(
+            $LayoutObject->Block(
                 Name => 'ContentSmallRSSTimeStamp',
                 Data => {
                     Ago   => $Ago,
@@ -127,7 +151,7 @@ sub Run {
             );
         }
         else {
-            $Self->{LayoutObject}->Block(
+            $LayoutObject->Block(
                 Name => 'ContentSmallRSS',
                 Data => {
                     Ago   => $Ago,
@@ -137,7 +161,7 @@ sub Run {
             );
         }
     }
-    my $Content = $Self->{LayoutObject}->Output(
+    my $Content = $LayoutObject->Output(
         TemplateFile => 'AgentDashboardRSSOverview',
         Data         => {
             %{ $Self->{Config} },
