@@ -1,6 +1,5 @@
 # --
-# Kernel/Output/HTML/TicketMenuGeneric.pm
-# Copyright (C) 2001-2016 OTRS AG, http://otrs.com/
+# Copyright (C) 2001-2017 OTRS AG, http://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
 # the enclosed file COPYING for license information (AGPL). If you
@@ -78,17 +77,7 @@ sub Run {
         ITEM:
         for my $Item (@Items) {
 
-            # get index for first colon (:) into the item
-            my $IndexSeparator = index( $Item, ':' );
-
-            # take the part for the permission from string
-            my $Permission = substr( $Item, 0, $IndexSeparator );
-
-            # increase the index for skipping the seperator
-            $IndexSeparator++;
-
-            # get the Group name
-            my $Name = substr( $Item, $IndexSeparator );
+            my ( $Permission, $Name ) = $Item =~ m{^([^:]+):(.*)$};
 
             if ( !$Permission || !$Name ) {
                 $Self->{LogObject}->Log(
