@@ -1,5 +1,5 @@
 # --
-# Copyright (C) 2001-2016 OTRS AG, http://otrs.com/
+# Copyright (C) 2001-2017 OTRS AG, http://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
 # the enclosed file COPYING for license information (AGPL). If you
@@ -17,18 +17,24 @@ use Digest::MD5 qw();
 use File::Path qw();
 use File::Find qw();
 
+our @ObjectDependencies = (
+    'Kernel::Config',
+    'Kernel::System::Encode',
+    'Kernel::System::Log',
+    'Kernel::System::Main',
+);
+
 sub new {
     my ( $Type, %Param ) = @_;
 
+    # allocate new hash for object
     my $Self = {};
     bless( $Self, $Type );
 
-    # check needed objects
-    for my $Needed (qw(ConfigObject LogObject MainObject EncodeObject)) {
-        $Self->{$Needed} = $Param{$Needed} || die "Got no $Needed!";
-    }
+    # get config object
+    my $ConfigObject = $Kernel::OM->Get('Kernel::Config');
 
-    my $TempDir = $Self->{ConfigObject}->Get('TempDir');
+    my $TempDir = $ConfigObject->Get('TempDir');
     $Self->{CacheDirectory} = $TempDir . '/CacheFileStorable';
 
     # check if cache directory exists and in case create one
@@ -37,7 +43,7 @@ sub new {
             ## no critic
             if ( !mkdir( $Directory, 0770 ) ) {
                 ## use critic
-                $Self->{LogObject}->Log(
+                $Kernel::OM->Get('Kernel::System::Log')->Log(
                     Priority => 'error',
                     Message  => "Can't create directory '$Directory': $!",
                 );
@@ -46,7 +52,7 @@ sub new {
     }
 
     # Specify how many levels of subdirectories to use, can be 0, 1 or more.
-    $Self->{'Cache::SubdirLevels'} = $Self->{ConfigObject}->Get('Cache::SubdirLevels');
+    $Self->{'Cache::SubdirLevels'} = $ConfigObject->Get('Cache::SubdirLevels');
     $Self->{'Cache::SubdirLevels'} //= 2;
 
     return $Self;
@@ -57,7 +63,7 @@ sub Set {
 
     for my $Needed (qw(Type Key Value TTL)) {
         if ( !defined $Param{$Needed} ) {
-            $Self->{LogObject}->Log(
+            $Kernel::OM->Get('Kernel::System::Log')->Log(
                 Priority => 'error',
                 Message  => "Need $Needed!"
             );
@@ -81,14 +87,14 @@ sub Set {
         File::Path::mkpath( $CacheDirectory, 0, 0770 );    ## no critic
 
         if ( !-e $CacheDirectory ) {
-            $Self->{LogObject}->Log(
+            $Kernel::OM->Get('Kernel::System::Log')->Log(
                 Priority => 'error',
                 Message  => "Can't create directory '$CacheDirectory': $!",
             );
             return;
         }
     }
-    my $FileLocation = $Self->{MainObject}->FileWrite(
+    my $FileLocation = $Kernel::OM->Get('Kernel::System::Main')->FileWrite(
         Directory  => $CacheDirectory,
         Filename   => $Filename,
         Content    => \$Dump,
@@ -98,7 +104,6 @@ sub Set {
     );
 
     return if !$FileLocation;
-
     return 1;
 }
 
@@ -108,7 +113,7 @@ sub Get {
     # check needed stuff
     for my $Needed (qw(Type Key)) {
         if ( !defined $Param{$Needed} ) {
-            $Self->{LogObject}->Log(
+            $Kernel::OM->Get('Kernel::System::Log')->Log(
                 Priority => 'error',
                 Message  => "Need $Needed!"
             );
@@ -118,7 +123,7 @@ sub Get {
 
     my ( $Filename, $CacheDirectory ) = $Self->_GetFilenameAndCacheDirectory(%Param);
 
-    my $Content = $Self->{MainObject}->FileRead(
+    my $Content = $Kernel::OM->Get('Kernel::System::Main')->FileRead(
         Directory       => $CacheDirectory,
         Filename        => $Filename,
         Type            => 'Local',
@@ -135,6 +140,7 @@ sub Get {
         $Self->Delete(%Param);
         return;
     }
+
     return $Storage->{Value};
 }
 
@@ -144,7 +150,7 @@ sub Delete {
     # check needed stuff
     for my $Needed (qw(Type Key)) {
         if ( !defined $Param{$Needed} ) {
-            $Self->{LogObject}->Log(
+            $Kernel::OM->Get('Kernel::System::Log')->Log(
                 Priority => 'error',
                 Message  => "Need $Needed!"
             );
@@ -154,7 +160,7 @@ sub Delete {
 
     my ( $Filename, $CacheDirectory ) = $Self->_GetFilenameAndCacheDirectory(%Param);
 
-    return $Self->{MainObject}->FileDelete(
+    return $Kernel::OM->Get('Kernel::System::Main')->FileDelete(
         Directory       => $CacheDirectory,
         Filename        => $Filename,
         Type            => 'Local',
@@ -165,11 +171,19 @@ sub Delete {
 sub CleanUp {
     my ( $Self, %Param ) = @_;
 
-    my @TypeList = $Self->{MainObject}->DirectoryRead(
+    # get main object
+    my $MainObject = $Kernel::OM->Get('Kernel::System::Main');
+
+    my @TypeList = $MainObject->DirectoryRead(
         Directory => $Self->{CacheDirectory},
         Filter    => $Param{Type} || '*',
     );
 
+    if ( $Param{KeepTypes} ) {
+        my $KeepTypesRegex = join( '|', map {"\Q$_\E"} @{ $Param{KeepTypes} } );
+        @TypeList = grep { $_ !~ m{/$KeepTypesRegex/?$}smx } @TypeList;
+    }
+
     return 1 if !@TypeList;
 
     my $FileCallback = sub {
@@ -184,7 +198,7 @@ sub CleanUp {
 
         # For expired filed, check the content and TTL
         if ( $Param{Expired} ) {
-            my $Content = $Self->{MainObject}->FileRead(
+            my $Content = $MainObject->FileRead(
                 Location        => $CacheFile,
                 Mode            => 'binmode',
                 DisableWarnings => 1,
@@ -200,7 +214,7 @@ sub CleanUp {
         # exist anymore, it was probably just another process deleting it.
         my $Success = unlink $CacheFile;
         if ( !$Success && $! != POSIX::ENOENT ) {
-            $Self->{LogObject}->Log(
+            $Kernel::OM->Get('Kernel::System::Log')->Log(
                 Priority => 'error',
                 Message  => "Can't remove file $CacheFile: $!",
             );
@@ -225,7 +239,7 @@ sub _GetFilenameAndCacheDirectory {
 
     for my $Needed (qw(Type Key)) {
         if ( !defined $Param{$Needed} ) {
-            $Self->{LogObject}->Log(
+            $Kernel::OM->Get('Kernel::System::Log')->Log(
                 Priority => 'error',
                 Message  => "Need $Needed!"
             );
@@ -234,7 +248,7 @@ sub _GetFilenameAndCacheDirectory {
     }
 
     my $Filename = $Param{Key};
-    $Self->{EncodeObject}->EncodeOutput( \$Filename );
+    $Kernel::OM->Get('Kernel::System::Encode')->EncodeOutput( \$Filename );
     $Filename = Digest::MD5::md5_hex($Filename);
 
     my $CacheDirectory = $Self->{CacheDirectory} . '/' . $Param{Type};
