@@ -1,6 +1,5 @@
 # --
-# Kernel/System/HTMLUtils.pm - creating and modifying html strings
-# Copyright (C) 2001-2016 OTRS AG, http://otrs.com/
+# Copyright (C) 2001-2017 OTRS AG, http://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
 # the enclosed file COPYING for license information (AGPL). If you
@@ -12,10 +11,13 @@ package Kernel::System::HTMLUtils;
 use strict;
 use warnings;
 
+use utf8;
+
 use MIME::Base64;
 
 our @ObjectDependencies = (
     'Kernel::Config',
+    'Kernel::System::Encode',
     'Kernel::System::Log',
 );
 
@@ -78,6 +80,9 @@ sub ToAscii {
         }
     }
 
+    # turn on utf8 flag (bug#10970, bug#11596 and bug#12097)
+    $Kernel::OM->Get('Kernel::System::Encode')->EncodeInput( \$Param{String} );
+
     # get length of line for forcing line breakes
     my $LineLength = $Self->{'Ticket::Frontend::TextAreaNote'} || 78;
 
@@ -480,9 +485,14 @@ sub ToAscii {
     }
     {
         my $Chr = chr( $2 );
-        # make sure we get valid UTF8 code points
-        Encode::_utf8_off( $Chr);
-        $Chr = Encode::decode('utf-8', $Chr, 0);
+
+        # Make sure we get valid UTF8 code points, but skip characters from 128 to 255
+        #   (inclusive), since they are by default internally not encoded as UTF-8 for
+        #   backward compatibility reasons. See bug#12457 for more information.
+        if ( $2 < 128 || $2 > 255 ) {
+            Encode::_utf8_off($Chr);
+            $Chr = Encode::decode('utf-8', $Chr, 0);
+        }
 
         if ( $Chr ) {
             $Chr;
@@ -498,12 +508,18 @@ sub ToAscii {
     }
     {
         my $ChrOrig = $1;
-        my $Hex = hex( $2 );
-        if ( $Hex ) {
-            my $Chr = chr( $Hex );
-            # make sure we get valid UTF8 code points
-            Encode::_utf8_off( $Chr);
-            $Chr = Encode::decode('utf-8', $Chr, 0);
+        my $Dec = hex( $2 );
+        if ( $Dec ) {
+            my $Chr = chr( $Dec );
+
+            # Make sure we get valid UTF8 code points, but skip characters from 128 to 255
+            #   (inclusive), since they are by default internally not encoded as UTF-8 for
+            #   backward compatibility reasons. See bug#12457 for more information.
+            if ( $Dec < 128 || $Dec > 255 ) {
+                Encode::_utf8_off($Chr);
+                $Chr = Encode::decode('utf-8', $Chr, 0);
+            }
+
             if ( $Chr ) {
                 $Chr;
             }
@@ -772,7 +788,7 @@ sub LinkQuote {
 
         # add target to existing "<a href"
         ${$String} =~ s{
-            (<a\s{1,10})(.+?)>
+            (<a\s{1,10})([^>]+)>
         }
         {
             my $Start = $1;
@@ -786,28 +802,30 @@ sub LinkQuote {
         }egxsi;
     }
 
-    # remove existing "<a href" on all other tags (to find not linked urls) and remember it
+    my $Marker = "§" x 10;
+
+    # Remove existing <a>...</a> tags and their content to be re-inserted later, this must not be quoted.
+    # Also remove other tags to avoid quoting in tag parameters.
     my $Counter = 0;
-    my %LinkHash;
+    my %TagHash;
     ${$String} =~ s{
-        (<a\s.+?>.+?</a>)
+        (<a\s[^>]*?>[^>]*</a>|<[^>]+?>)
     }
     {
         my $Content = $1;
-        $Counter++;
-        my $Key  = "############LinkHash-$Counter############";
-        $LinkHash{$Key} = $Content;
+        my $Key     = "${Marker}TagHash-$Counter${Marker}";
+        $TagHash{$Counter++} = $Content;
         $Key;
     }egxism;
 
-    # replace not "<a href" found urls and link it
+    # Add <a> tags for URLs in the content.
     my $Target = '';
     if ( $Param{Target} ) {
         $Target = " target=\"$Param{Target}\"";
     }
     ${$String} =~ s{
         (                                          # $1 greater-than and less-than sign
-            > | < | \s+ | \#{6} |
+            > | < | \s+ | §{10} |
             (?: &[a-zA-Z0-9]+; )                   # get html entities
         )
         (                                          # $2
@@ -844,7 +862,7 @@ sub LinkQuote {
                 | (?: &[a-zA-Z0-9]+; )+            # html entities
                 | $                                # bug# 2715
             )
-            | \#{6}                                # ending LinkHash
+            | §{10}                                # ending TagHash
         )
     }
     {
@@ -880,12 +898,8 @@ sub LinkQuote {
         $Start . "<a href=\"$HrefLink\"$Target title=\"$HrefLink\">$DisplayLink<\/a>" . $End;
     }egxism;
 
-    my ( $Key, $Value );
-
-    # add already existing "<a href" again
-    while ( ( $Key, $Value ) = each(%LinkHash) ) {
-        ${$String} =~ s{$Key}{$Value};
-    }
+    # Re-add previously removed tags.
+    ${$String} =~ s{${Marker}TagHash-(\d+)${Marker}}{$TagHash{$1}}egsxim;
 
     # check ref && return result like called
     if ($StringScalar) {
